use crate::socket::{create_niri_socket, tell};
use anyhow::Result;
use niri_ipc::{Event, Request};
use niri_tag::TagCmd;
use nix::unistd::geteuid;
use smol::{
    channel::{self},
    io::{AsyncBufReadExt, AsyncWriteExt, BufReader},
    net::unix::UnixListener,
};

#[allow(unreachable_code)]
pub async fn event_listener(tx: channel::Sender<Event>) -> Result<()> {
    tracing::debug!("creating listener socket");
    let mut socket = create_niri_socket().await?;
    tracing::debug!("requesting event stream");
    let mut buf = String::new();
    tell(&mut socket, Request::EventStream).await?;
    tracing::debug!("beginning event loop");
    loop {
        let _ = socket.read_line(&mut buf).await?;
        let event: Event = serde_json::from_str(&buf)?;
        tracing::debug!("event: {:?}", event);
        tx.send(event).await?;
    }
    unreachable!("Listener loop ended");
}

#[allow(unreachable_code)]
pub async fn ipc_listener(tx: channel::Sender<TagCmd>) -> Result<()> {
    tracing::debug!("creating niri-tag socket");
    let sock_path = format!("/run/user/{}/niri-tag.sock", geteuid());
    if smol::fs::metadata(&sock_path).await.is_ok() {
        tracing::debug!("removing old niri-tag socket");
        smol::fs::remove_file(&sock_path).await?;
    }
    tracing::debug!("establishing niri-tag socket connection");
    let listen = UnixListener::bind(&sock_path)
        .inspect_err(|f| tracing::error!("failed to listen to niri-tag socket: {}", f))?;
    let mut buf = String::new();
    loop {
        let (raw_sock, _addr) = listen.accept().await?;
        let mut socket = BufReader::new(raw_sock);
        tracing::debug!("awaiting ipc socket");
        let _ = socket.read_line(&mut buf).await?;
        tracing::debug!("forwarding ipc command {}", buf);
        tx.send(serde_json::from_str(&buf)?).await?;
        socket.close().await?;
    }
    tracing::error!("IPC loop ended");
    unreachable!("IPC loop ended");
    Ok(())
}
